from docassemble.base.util import CustomDataType, DAObject, get_config, log
import requests

__all__ = ['SpotResult']

api_root = get_config('spot api url', 'https://spot.suffolklitlab.org/v0')

class SpotResult(DAObject):
    @property
    def id(self):
      if self.result is None:
        return None
      if 'id' not in self._full_result['labels'][0]:
        log('Spot API returned a result but it lacked an id')
        return None
      return self._full_result['labels'][0]['id']
    @property
    def result(self):
        if hasattr(self, '_active_result') and hasattr(self, '_active_source') and self._active_source == self.source:
            return self._active_result
        if hasattr(self, '_active_result'):
            del self._active_result
        if hasattr(self, '_full_result'):
            del self._full_result
        self._active_source = self.source
        api_key = get_config('spot api key')
        if api_key is None:
            raise Exception("Cannot run Spot api without a 'spot api key' directive in the Configuration.")
        input_data = {
            "text": self._active_source,
            "save-text": 0,
            "cutoff-lower": 0.25,
            "cutoff-pred": 0.5,
            "cutoff-upper": 0.5
        }
        headers = {"Authorization": "Bearer " + api_key, "Content-Type": "application/json", "Accept": "application/json"}
        r = requests.post(api_root + '/entities-nested/', json=input_data, headers=headers)
        success = True
        if r.status_code != 200:
            success = False
            log('Spot API returned non-success status code ' + str(r.status_code) + "\n" + str(r.text))
        if success:
            try:
                self._full_result = r.json()
            except:
                success = False
                log('Spot API did not return valid JSON')
        if success and ('labels' not in self._full_result or len(self._full_result['labels']) == 0):
            success = False
            log('Spot API returned no results')
        if success and 'name' not in self._full_result['labels'][0]:
            success = False
            log('Spot API returned a result but it lacked a name')
        if success:
            self._active_result = self._full_result['labels'][0]['name']
        else:
            self._active_result = None
        return self._active_result
    def __str__(self):
        return self.result

class Spot(CustomDataType):
    name = 'spot'
    container_class = 'da-spot-container'
    input_class = 'da-spot'
    is_object = True
    @classmethod
    def transform(cls, item):
        new_item = SpotResult()
        new_item.source = item
        return new_item
    @classmethod
    def default_for(cls, item):
        return item.source


import os
import sys
from setuptools import setup, find_packages
from fnmatch import fnmatchcase
from distutils.util import convert_path

standard_exclude = ('*.pyc', '*~', '.*', '*.bak', '*.swp*')
standard_exclude_directories = ('.*', 'CVS', '_darcs', './build', './dist', 'EGG-INFO', '*.egg-info')
def find_package_data(where='.', package='', exclude=standard_exclude, exclude_directories=standard_exclude_directories):
    out = {}
    stack = [(convert_path(where), '', package)]
    while stack:
        where, prefix, package = stack.pop(0)
        for name in os.listdir(where):
            fn = os.path.join(where, name)
            if os.path.isdir(fn):
                bad_name = False
                for pattern in exclude_directories:
                    if (fnmatchcase(name, pattern)
                        or fn.lower() == pattern.lower()):
                        bad_name = True
                        break
                if bad_name:
                    continue
                if os.path.isfile(os.path.join(fn, '__init__.py')):
                    if not package:
                        new_package = name
                    else:
                        new_package = package + '.' + name
                        stack.append((fn, '', new_package))
                else:
                    stack.append((fn, prefix + name + '/', package))
            else:
                bad_name = False
                for pattern in exclude:
                    if (fnmatchcase(name, pattern)
                        or fn.lower() == pattern.lower()):
                        bad_name = True
                        break
                if bad_name:
                    continue
                out.setdefault(package, []).append(prefix+name)
    return out

setup(name='docassemble.spot',
      version='0.0.2',
      description=('Provides a datatype for legal issue spotting using the Spot API.'),
      long_description="This package provides a `datatype` for mapping descriptions of legal problems\r\nto [NSMIv2] categories.  It uses the [Spot API] developed by the\r\n[The Legal Innovation & Technology Lab] at [Suffolk Law School].\r\n\r\nThis package requires **docassemble** version 0.5.86 or later.\r\n\r\nTo use package, first [obtain an API key] for the [Spot API].\r\n\r\nThen put that API key into your [Configuration]:\r\n\r\n```\r\nspot api key: abbaabba1234abbaabba1234abbaabba1234abbaabba1234abbaabba\r\n```\r\n\r\nThen you can use `spot` as a `datatype`.  For example:\r\n\r\n```\r\nquestion: |\r\n  What is your legal issue?\r\nfields:\r\n  - no label: legal_issue\r\n    input type: area\r\n    datatype: spot\r\n---\r\nmandatory: True\r\nquestion: |\r\n  % if legal_issue.result == 'Housing':\r\n  We can help you with that housing issue.\r\n  % else:\r\n  Sorry, we don't help with that.\r\n  % endif\r\n```\r\n\r\nIn this example, the variable `legal_issue` will become an object of type\r\n`SpotResult`.  This is a subclass of `DAObject`.  The user's original text is\r\navailable at `legal_issue.source`.  The result is available at `legal_issue.result`.\r\nWhen reduced to text, a `SpotResult` object returns `legal_issue.result`.  If the \r\nlegal issue cannot not be determined, `legal_issue` will be `None` and an error\r\nmessage will be written to the logs.  The [NSMIv2] code is available under `legal_issue.id`.\r\n\r\nOnly the first result is used for the `.result` and `.id` attributes.  If you \r\nwant to inspect into the actual result returned by the API, you can find it \r\nunder `legal_issue._full_result`.\r\n\r\n[Spot API]: https://spot.suffolklitlab.org/\r\n[Configuration]: https://docassemble.org/docs/config.html\r\n[obtain an API key]: https://spot.suffolklitlab.org/user/new/\r\n[The Legal Innovation & Technology Lab]: https://suffolklitlab.org/\r\n[Suffolk Law School]: https://www.suffolk.edu/law\r\n[NSMIv2]: http://betterinternet.law.stanford.edu/about-the-project/legal-issues-taxonomy-nsmiv2/",
      long_description_content_type='text/markdown',
      author='Jonathan Pyle',
      author_email='jhpyle@gmail.com',
      license='The MIT License (MIT)',
      url='https://docassemble.org',
      packages=find_packages(),
      namespace_packages=['docassemble'],
      install_requires=[],
      zip_safe=False,
      package_data=find_package_data(where='docassemble/spot/', package='docassemble.spot'),
     )

